/**  Simple MVC framework 
  *
  *  redis pubsub built in
  *  automatically loads views
  *  automatically loads view rendering engine
  */

var _ = require('underscore'),
hash = require('hashish'),
walk = require('./view.walk');

db = {};
publish = {};
subscribe = {};

exports.Model = require('./model');
exports.View = require('./view');
exports.Controller = require('./controller');
exports.middleware = require('./middleware');

exports.database = {
    type: 'memory',
    args: [],
    connections: []
};

exports['template engine'] = 'ejs';
exports.templates = [];

exports.set = function () {
    if (typeof arguments[0] !== 'string') return;
    var key = arguments[0];
    var value = arguments[1];
    var params = arguments[2];
    switch (key) {
        case 'debug':
            console.log('Tubes in debug mode.');
            exports.debug = value;
            exports.Model.debug = value;
            exports.View.debug = value;
            exports.Controller.debug = value;
            exports.middleware.debug = value;
        break;
        case 'database':
            exports.database.type = value;
            exports.database.args = params;
            if (exports.debug) console.log('setting database', exports.database);
            var file = require(value);
            if (value === 'redis') {
                publish = exports.publish = file.createClient.call(params);
                subscribe = exports.subscribe = file.createClient.call(params);
                exports.subscribe.on('message', function(stream, data) {
                    new hash(exports.clients[stream]).forEach(function (emit) {
                        emit.call({}, JSON.parse(data));
                    });
                });
                
                var file = require(value);
                db = exports.db = file.createClient.call(params);
                
            }
            if (value === 'mongodb') exports.db = exports.db.connect(params);
        break;
        case 'template engine':
            if (exports.debug) console.log('setting template engine', value);
            exports['template engine'] = value;
            exports[value] = require(value);
            exports.render = exports[value].render;
        break;
        case 'templates directory':
            if (exports.debug) console.log('setting templates directory', value);
            var dir = arguments[1] || 'views';
            walk('./' + dir, function (e, tpls) {
                tpls = tpls || [];
                if (exports.debug) console.log('Loaded ' + Object.keys(tpls).length + ' views from ' + dir);
                exports.templates = tpls;
            });
        break;
    }
};

exports.clients = {};

exports.removeStream = function (stream, conn) {
    conn.streams.forEach(function (stream) {
        if (stream && conn.id && exports.clients[stream]) {
            delete exports.clients[stream][conn.id];
            conn.streams.splice(conn.streams.indexOf(stream), 1);
        }
        if (stream) {
            subscribe.unsubscribe(stream);
        }
    });
};

exports.registerStream = function (stream, conn, callback) {
    conn.streams = conn.streams || [];
    conn.streams.push(stream);
    
    console.log(conn.streams);
    
    if(!exports.clients[stream]) {
        exports.clients[stream] = {};
    }
    exports.clients[stream][conn.id] = callback;
    exports.subscribe.subscribe(stream);
    
    conn.on('end', function () {
        exports.removeStream(stream, conn);
    });
};

